      subroutine svd(mdim,ndim,m,n,a,w,u,v,temp,ierr)                   
      integer mdim,ndim,m,n,ierr                                        
      real a(mdim,n),w(n),u(mdim,n),v(ndim,n),temp(m)                   
c-----------------------------------------------------------------------
c     this subroutine determines the singular value decomposition       
c          t                                                            
c     a=usv  of a real m-by-n rectangular matrix. it uses the           
c     householder bidiagonalization and the qr algorithms.              
c                                                                       
c     subroutine arguments:                                             
c        mdim must be set to the row dimension of a and u.              
c        ndim must be set to the row dimension of v.                    
c        m is the number of rows of a (and u); m must be >=n.           
c        n is the number of columns of a (and u) and the order of v.    
c        a contains the input matrix to be decomposed; a will not be    
c          altered in svd.                                              
c        w contains the n (non-negative) singular values of a (the      
c           diagonal elements of s) in nonincreasing order. if an error 
c           exit is made, the singular values should be correct         
c           for indices ierr+1,ierr+2,...,n, and they are unordered.    
c        u contains the matrix u (orthogonal column vectors) of         
c           the decomposition. if an error exit is made, the columns    
c           of u corresponding to indices of correct singular values    
c           should be correct.                                          
c        v contains the matrix v (orthogonal) of the decomposition.     
c           if an error exit is made, the columns of v corresponding    
c           to indices of correct singular values should be correct.    
c        temp is a temporary storage array & must be provided by user.  
c        ierr is set to                                                 
c           zero     for normal return,                                 
c           k        if the k-th singular value has not been            
c                    determined after 30 iterations.                    
c-----------------------------------------------------------------------
c     this subroutine is a modified version of the subroutine svd       
c     in the book 'computer methods for mathematical computations,'     
c     by forsythe, malcolm, and moler.                                  
c     modifications by w.h.k. lee & f. luk (1/26/78; 5/30/79).          
c-----------------------------------------------------------------------
      integer i,j,k,l,ii,i1,kk,k1,ll,l1,mn,its                          
      real anorm,f,p,q                                                  
      double precision c,g,h,s,x,y,z,scale                              
      double precision dble,dsqrt,dsign                                 
      ierr = 0                                                          
      do 100 i = 1, m                                                   
         do 100 j = 1, n                                                
  100       u(i,j) = a(i,j)                                             
c--------------- householder reduction to bidiagonal form --------------
      g = 0.0d0                                                         
      scale = 0.0d0                                                     
      anorm = 0.                                                        
      do 300 i = 1, n                                                   
         l = i + 1                                                      
         temp(i) = scale * g                                            
         g = 0.0d0                                                      
         s = 0.0d0                                                      
         scale = 0.0d0                                                  
         if (i .gt. m) go to 210                                        
         do 120 k = i, m                                                
  120       scale = scale + abs(u(k,i))                                 
         if (scale .eq. 0.0d0) go to 210                                
         do 130 k = i, m                                                
            u(k,i) = u(k,i) / scale                                     
  130       s = s + dble(u(k,i))**2                                     
         f = u(i,i)                                                     
         g = -dsign(dsqrt(s),dble(f))                                   
         h = f * g - s                                                  
         u(i,i) = f - g                                                 
         if (i .eq. n) go to 190                                        
         do 150 j = l, n                                                
            s = 0.0d0                                                   
            do 140 k = i, m                                             
  140          s = s + dble(u(k,i)) * dble(u(k,j))                      
            f = s / h                                                   
            do 145 k = i, m                                             
  145          u(k,j) = u(k,j) + f * u(k,i)                             
  150       continue                                                    
  190    do 200 k = i, m                                                
  200       u(k,i) = scale * u(k,i)                                     
  210    w(i) = scale * g                                               
         g = 0.0d0                                                      
         s = 0.0d0                                                      
         scale = 0.0d0                                                  
         if (i .gt. m .or. i .eq. n) go to 290                          
         do 220 k = l, n                                                
  220       scale = scale + abs(u(i,k))                                 
         if (scale .eq. 0.0d0) go to 290                                
         do 230 k = l, n                                                
            u(i,k) = u(i,k) / scale                                     
  230       s = s + dble(u(i,k))**2                                     
         f = u(i,l)                                                     
         g = -dsign(dsqrt(s),dble(f))                                   
         h = f * g - s                                                  
         u(i,l) = f - g                                                 
         do 240 k = l, n                                                
  240       temp(k) = u(i,k) / h                                        
         if (i .eq. m) go to 270                                        
         do 260 j = l, m                                                
            s = 0.0d0                                                   
            do 250 k = l, n                                             
  250          s = s + dble(u(j,k)) * dble(u(i,k))                      
            do 255 k = l, n                                             
  255          u(j,k) = u(j,k) + s * temp(k)                            
  260       continue                                                    
  270    do 280 k = l, n                                                
  280       u(i,k) = scale * u(i,k)                                     
  290    anorm = amax1(anorm,abs(w(i))+abs(temp(i)))                    
  300    continue                                                       
c--------------- accumulation of right-hand transformations ------------
      do 400 ii = 1, n                                                  
         i = n + 1 - ii                                                 
         if (i .eq. n) go to 390                                        
         if (g .eq. 0.0d0) go to 360                                    
         do 320 j = l, n                                                
  320       v(j,i) = (u(i,j) / u(i,l)) / g                              
         do 350 j = l, n                                                
            s = 0.0d0                                                   
            do 340 k = l, n                                             
  340          s = s + dble(u(i,k)) * dble(v(k,j))                      
            do 345 k = l, n                                             
  345          v(k,j) = v(k,j) + s * v(k,i)                             
  350       continue                                                    
  360    do 380 j = l, n                                                
            v(i,j) = 0.                                                 
  380       v(j,i) = 0.                                                 
  390    v(i,i) = 1.                                                    
         g = temp(i)                                                    
         l = i                                                          
  400    continue                                                       
c--------------- accumulation of left-hand transformations -------------
      mn = n                                                            
      if (m .lt. n) mn = m                                              
      do 500 ii = 1, mn                                                 
         i = mn + 1 - ii                                                
         l = i + 1                                                      
         g = w(i)                                                       
         if (i .eq. n) go to 430                                        
         do 420 j = l, n                                                
  420       u(i,j) = 0.                                                 
  430    if (g .eq. 0.0d0) go to 475                                    
         if (i .eq. mn) go to 460                                       
         do 450 j = l, n                                                
            s = 0.0d0                                                   
            do 440 k = l, m                                             
  440       s = s + dble(u(k,i)) * dble(u(k,j))                         
            f = (s / u(i,i)) / g                                        
            do 445 k = i, m                                             
  445          u(k,j) = u(k,j) + f * u(k,i)                             
  450       continue                                                    
  460    do 470 j = i, m                                                
  470       u(j,i) = u(j,i) / g                                         
         go to 490                                                      
  475    do 480 j = i, m                                                
  480       u(j,i) = 0.                                                 
  490    u(i,i) = u(i,i) + 1.                                           
  500    continue                                                       
c--------------- diagonalization of the bidiagonal form ----------------
      do 700 kk = 1, n                                                  
         k1 = n - kk                                                    
         k = k1 + 1                                                     
         its = 0                                                        
c--------------- test for splitting ------------------------------------
  520    do 530 ll = 1, k                                               
            l1 = k - ll                                                 
            l = l1 + 1                                                  
            if ( abs(temp(l))+anorm .eq. anorm) go to 565               
c----- temp(1) is always zero, so there is no exit thru the loop bottom-
  530       continue                                                    
            if ( abs(w(l1))+anorm .eq. anorm) go to 540                 
c--------------- cancellation of temp(l) if l greater than 1 -----------
  540    c = 0.0d0                                                      
         s = 1.0d0                                                      
         do 560 i = l, k                                                
            f = s * temp(i)                                             
            temp(i) = c * temp(i)                                       
            if ( abs(f)+anorm .eq. anorm) go to 565                     
            g = w(i)                                                    
            h = dsqrt( dble(f)*dble(f) + g*g )                          
            w(i) = h                                                    
            c = g / h                                                   
            s = -f / h                                                  
            do 550 j = 1, m                                             
               y = u(j,l1)                                              
               z = u(j,i)                                               
               u(j,l1) = y * c + z * s                                  
  550          u(j,i) = -y * s + z * c                                  
  560       continue                                                    
c--------------- test for convergence ----------------------------------
  565    z = w(k)                                                       
         if (l .eq. k) go to 650                                        
c--------------- shift from bottom 2 by 2 minor ------------------------
c         if (its .eq. 30) go to 1000                                    
c         if (its .eq. 35) go to 1000                                    
         its = its + 1                                                  
         x = w(l)                                                       
         y = w(k1)                                                      
         g = temp(k1)                                                   
         h = temp(k)                                                    
         f = ((y - z) * (y + z) + (g - h) * (g + h)) / (2.0d0 * h * y)  
         g = dsqrt( dble(f)*dble(f) + 1.0d0 )                           
         f = ((x - z)*(x + z) + h*(y/(f + dsign(g,dble(f))) - h))/x     
c--------------- next qr transformation --------------------------------
         c = 1.0d0                                                      
         s = 1.0d0                                                      
         do 600 i1 = l, k1                                              
            i = i1 + 1                                                  
            g = temp(i)                                                 
            y = w(i)                                                    
            h = s * g                                                   
            g = c * g                                                   
            z = dsqrt( dble(f)*dble(f) + h*h )                          
            temp(i1) = z                                                
            c = f / z                                                   
            s = h / z                                                   
            f = x * c + g * s                                           
            g = -x * s + g * c                                          
            h = y * s                                                   
            y = y * c                                                   
            do 570 j = 1, n                                             
               x = v(j,i1)                                              
               z = v(j,i)                                               
               v(j,i1) = x * c + z * s                                  
  570          v(j,i) = -x * s + z * c                                  
            z = dsqrt( dble(f)*dble(f) + h*h )                          
            w(i1) = z                                                   
c--------------- rotation can be arbitrary if z is zero ----------------
            if (z .eq. 0.0d0) go to 580                                 
            c = f / z                                                   
            s = h / z                                                   
  580       f = c * g + s * y                                           
            x = -s * g + c * y                                          
            do 590 j = 1, m                                             
               y = u(j,i1)                                              
               z = u(j,i)                                               
               u(j,i1) = y * c + z * s                                  
  590          u(j,i) = -y * s + z * c                                  
  600       continue                                                    
         temp(l) = 0.0                                                  
         temp(k) = f                                                    
         w(k) = x                                                       
         go to 520                                                      
c--------------- convergence -------------------------------------------
  650    if (z .ge. 0.0d0) go to 700                                    
c--------------- w(k) is made non-negative -----------------------------
         w(k) = -z                                                      
         do 690 j = 1, n                                                
  690       v(j,k) = -v(j,k)                                            
  700    continue                                                       
c-----arrange singular values in non-increasing order & order u & v also
  805 do 900 k = 1,n                                                    
         p = -1.0                                                       
         do 810 i = k,n                                                 
            if (w(i) .lt. p) go to 810                                  
            p = w(i)                                                    
            j = i                                                       
  810       continue                                                    
         if (j .eq. k) go to 900                                        
         w(j) = w(k)                                                    
         w(k) = p                                                       
         do 820 i = 1,n                                                 
            q = v(i,j)                                                  
            v(i,j) = v(i,k)                                             
  820       v(i,k) = q                                                  
         do 830 i = 1,m                                                 
            q = u(i,j)                                                  
            u(i,j) = u(i,k)                                             
  830       u(i,k) = q                                                  
  900    continue                                                       
      return                                                            
c----- set error -- no convergence to singular value after 30 iterations
 1000 ierr = k                                                          
      return                                                            
      end                                                               
